{$o-} {this unit may >>>>not<<<< be overlaid}

{$x+} {extended syntax is ok}

{$f+} {allow far calls}

{$a+} {word align data}

{$g+} {286 code optimization - if you're using an 8088 get a real computer}

{$R-} {Disable range checking}

{$S+} {Enable Stack Checking}

{$I-} {Disable IO Checking}

{$Q-} {Disable Overflow Checking}

{$D-} {Turn off debugging - use only if you modify this unit and get a bug}

{$L-} {Turn off local symbols - again this unit has been thouroughly debuged}

{$V-} {Turn off strict VAR strings}

{$B-} {Allow short circuit boolean evaluations}

{$T-} {Turn off typed @ operators}



Unit Async;



{This unit was written by Patrick Hunlock, 10/19/1994     }

{Copyright @ 1994 by Patrick Hunlock - all rights reserved}



{Software License Agreement: If you use this unit you are bound by it's terms}



{you may use this unit in your programs without royalties. if you use this

 unit my name and the copyright notice must appear beneath your name and

 copyright notice even if you have made significant alterations to the

 source code in this unit.  this source code may only be transmitted via

 the telephone system.  it may not be distributed on any magnetic, optical,

 or any future computer media format without prior consent of the copyright

 holder.  no charge may ever be levied for the receipt of this source code

 other than normal phone charges and/or normal connect charges on a bbs which

 charges for access.  any charges above and beyond what would be considered

 normal access charges to download this source is considered a sale of this

 source code which is expressly prohibited by this license agreement.

 you may distribute this source code via the telephone system (i.e.

 bbs or computer network) in it's original form only, you may not add or

 subtract to/from it in any way shape or form.}



{This ASYNC unit implements a >COMPLETE< multi-port serial interface wrapped

 in a turbo pascal object.  Baud rates up to 110,000 are supported.  Up to

 four ports may be opened at a time, up to two ports may be active at any

 given time (four can be active so long as they do not share interrupts).

 Some considerations.  A com port which shares the same interrupt as a

 serial mouse will over-ride the mouse driver for the duration of the

 program.  If you have a mouse on Com1, avoid using Com3 in your program.

 Likewise if you have a mouse on Com2, avoid using Com4 in your program.

 A mouse on a BUS card will not be affected by this unit.  Default addresses

 for the 4 com ports dictate that com1 & com3, and com2 & com4 share an

 interrupt on the system bus.  As such while you can open both ports, make

 sure that only - ONE of the ports on the shared interrupt has been .ENABLED

 at any one time.  See the .ENABLE and .DISABLE procedures for more

 information.}



Interface



Type



   Com_Port      = Object

                         CPort   : Byte;          {Com Port for this port}



                      {Initializes the buffers and the object}

                      Function Init(ComPort: Byte; RBufSize,TBufSize: Word): Byte;

                      {Sets the baud,parity,wordsize, and stopbits}

                      Procedure SetParam(Baud: Longint; WordSize: Byte;

                                         Parity: Char; StopBits: Byte);

                      {Used for shared interrupts}

                      Procedure Disable;

                      {Enable a com port for use}

                      Procedure Enable;

                      {Returns true if there is data waiting}

                      Function Waiting: Boolean;

                      {Returns a character waiting in the buffer}

                      Function Read: Char;

                      {Waits for a character if necessary}

                      Function Readw: Char;

                      {Writes a character to the transmit buffer}

                      Procedure Write(C: CHar);

                      {Passes a string to the transmit buffer}

                      Procedure WriteS(S: String);

                      {Returns true if modem is "on-line" re DCD status}

                      Function OnLine: Boolean;

                      {Disconnects the modem connection}

                      Procedure Hangup;

                      {Sends a Break Signal to the other computer}

                      Procedure Break;

                      {Terminates the comm port}

                      Procedure Done;

                   End;



Implementation



Uses DOS,       {Turbo Pascal's dos unit}

     crt;       {turbo pascal's CRT Unit}



Const

                                     {Com1,Com2,Com3,Com4}

   PortBases : Array[1..4] Of Word = ($3F8,$2F8,$3E8,$2E8);

   Interrupts: Array[1..4] Of Byte = (   4,   3,   4,   3);



   Disable_Interrupts = $FA;       {Used in INLINE statements}

   Enable_Interrupts  = $FB;       {Used in INLINE statements}



Type

   {Buf Type is never actually >declared< in this unit.  It is created only

    as a pointer to an area of memory which can actually only be 10 to 64000

    bytes long.  However by creating the pointer in this way, the contents

    of the pointer memory can be accessed like an array of chars simplifying

    the coding of this unit.  If you only create a 10000 byte buffer then

    it's valid to read buftype[0] through buftype[10000] anything higher

    is dangerous since no memory was set aside for this buffer.  however

    the unit knows how much memory was set asside and it never references

    any data outside of the "safe" range}

   buftype = array[0..64000] of char;



   {combuffertype is the actual buffer used by the com_isr routine and the

    record which actually makes com_port work.  in reality com_port is just

    a shell which is wrapped around this record to give the illusion of

    oop.  take away com_port, write your own procedures to reference

    combuffertype and your program will work just fine}



   combuffertype = record

                      active       : boolean;  {true if this buffer is active}

                      r_buffer     : ^buftype; {recieve buffer pointer       }

                      r_head,r_tail: word;     {buffer head and buffer tail  }

                      r_size       : word;     {size of the recieve buffer   }

                      t_buffer     : ^buftype; {transmit buffer pointer      }

                      t_head,t_tail: word;     {transmit buffer head & tail  }

                      t_size       : word;     {size of the transmit buffer  }

                      uart_data    : word;     {uart data address            }

                      uart_ier     : word;     {uart interrupt enable registr}

                      uart_iir     : word;     {uart interrupt identification}

                      uart_lcr     : word;     {uart line control register   }

                      uart_mcr     : word;     {uart modem control register  }

                      uart_lsr     : word;     {uart line status register    }

                      uart_msr     : word;     {uart modem status register   }

                      old_mcr      : byte;     {old modem control register   }

                      org_vector   : pointer;  {original interrupt vector    }

                   end;



var

   bufs: array[1..4] of combuffertype; {this declares 4 com buffers for  }

                                       {coms 1 - 4                       }



procedure com_isr; interrupt;



{com_isr is the main interrupt procedure which handles all the serial io.

 this procedure is called >automatically< by dos whenever data arrives at

 the com port - or when it is clear to send data.  >severe< restrictions

 as to what you can and can not add to this procedure apply.  you can not

 use writeln.  you can not reference any turbo pascal objects.  this unit

 may not be overlaid.  etc, etc, etc}



const ktr: byte = 0;  {these are static variables so pascal doesn't }

      IIR: Byte = 0;  {constantly have to redeclare them on the heap}



Begin

   For Ktr:= 1 to 4 Do begin

      With Bufs[Ktr] Do Begin

         If Active Then Begin

            iir:= Port[UART_IIR];

            While Not Odd(IIR) Do Begin

               Case (iir SHR 1) Of

                  0: iir:= Port[UART_MSR]; {Modem status change}

                  1: If T_Head = T_Tail Then Begin    {Ok to transmit      }

                        {Transmit buffer empty - disable transmit interrupt}

                        Port[UART_IER]:= Port[UART_IER] And Not 2;

                     End Else Begin

                        Port[UART_DATA]:= Byte(T_Buffer^[T_Head]);

                        Inc(T_Head);

                        If T_Head > T_Size Then T_Head:= 0;

                     End;

                  2: Begin                  {Recieve buffer}

                        R_Buffer^[R_Tail]:= Char(Port[Uart_Data]);

                        Inc(R_Tail);

                        If R_Tail > R_Size Then R_Tail:= 0;

                        If (R_Tail = R_Head) Then Begin

                           Inc(R_Head); {Overflow}

                           If R_Head > R_Size Then R_Head:= 0;

                        End;

                     End;

                  3: iir:= Port[UART_LSR]; {Line status change}

               End;

               iir:= Port[UART_IIR];

            End;

         End;

      End;

   End;

   Port[$20]:= $20;  {We're done processing the interrupt}

end;



function com_port.init(comport: byte; rbufsize,tbufsize: word): byte;



{init is the standard object initialization routine. you must pass the

 comport (1-4) you want the object to be associated with and the buffer

 size (10-64000 bytes) you want the buffer to be.  init returns the following

 codes based upon it's success or failure......



    0 - Com Port Initialized

    1 - ComPort is out of range - not 1, 2, 3, or 4.

    2 - ComPort is already active and in use.

    3 - Buffer size is either to small (10 bytes or more) or to large

        (greater than 64000) - Recieve Buffer

    4 - Transmit buffer size out of range (See #3)



 Init only sets up the buffers and prepares the interrupt vector.  You must

 follow this with a call to setparams (set baud rate, parity, etc), then

 a call to enable.  See enable and disable especially if you are going to

 use multiple comm ports simultaniously.

}



Var InUse: Boolean;     {Scratch variable to check for active interrupts}

    Ktr  : Byte;        {Counter Variable                               }



Begin

   {Set the initial state of the return code to OK}

   Init:= 0;

   {Check the comport validity}

   If (ComPort < 1) Or (ComPort > 4) Then Begin

      Init:= 1;

      Exit;

   End;

   {Check to see if the comport is already in use by another object}

   If Bufs[ComPort].Active Then Begin

      Init:= 2;

      Exit;

   End;

   {Check to make sure the buffer size is valid}

   If (RBufSize > 64000) Or (RBufSize < 10) Then Begin

      Init:= 3;

      Exit;

   End;

   If (TBufSize > 64000) Or (TBufSize < 10) Then Begin

      Init:= 4;

      Exit;

   End;



   {Begin main setup}



   CPort:= ComPort;                        {Store the comport for future use}

   Getmem(Bufs[ComPort].R_Buffer,RBufSize);{Allocate memory for the buffer  }

   Bufs[ComPort].R_Size:= RBufSize;        {Store the size of the buffer    }

   GetMem(Bufs[ComPort].T_Buffer,TBufSize);{Allocate transmit buffer memory }

   Bufs[ComPort].T_Size:= TBufSize;        {Store the size of the buffer    }



   {This next section sets up the PORT addresses used by the comport

    requested.  The base addresses are stored in PORTBASES, a constant

    declared at the top of the implenetation section of this unit.  Your

    program may need to change the address and/or interrupts found in

    that section for serial cards with unusual addresses and interrupts.

    Since PortBases and Interrupts are typed constant arrays you can

    easilly add an object method to change the address or interrupt for

    a given comm port}



    Bufs[ComPort].UART_DATA:= PortBases[ComPort]+0;

    Bufs[ComPort].UART_IER := PortBases[ComPort]+1;

    Bufs[ComPort].UART_IIR := PortBases[ComPort]+2;

    Bufs[ComPort].UART_LCR := PortBases[ComPort]+3;

    Bufs[ComPort].UART_MCR := PortBases[ComPort]+4;

    Bufs[ComPort].UART_LSR := PortBases[ComPort]+5;

    Bufs[ComPort].UART_MSR := PortBases[ComPort]+6;





   {This next section sees if there is already an interrupt vector set for

    a shared interrupt com port.  For instance COM1 and COM3 both use

    interrupt 4 and COM2 and COM4 use interrupt 3.  If we are setting up

    COM1, but COM3 is already up and running we don't need to do anything

    since the interrupt service routine (procedure com_isr) will

    automatically check the com ports on it's interrupt}



   InUse:= False;

   For Ktr:= 1 to 4 Do

      If (Interrupts[Ktr] = Interrupts[ComPort]) And Bufs[Ktr].Active Then

         InUse:= True;



   {This next section is run if, and only if a shared interrupt is not

    currently running.}



   InLine(Disable_Interrupts);



   If Not InUse Then Begin

      {Get the old DOS interrupt vector, save it then change it to point

       to the COM_ISR procedure in this unit}

      Port[$21] := Port[$21] Or (1 SHL Interrupts[ComPort]);

      GetIntVec(8+Interrupts[ComPort],Bufs[ComPort].Org_Vector);

      SetIntVec(8+Interrupts[ComPort],@COM_ISR);

      Port [$21] := Port [$21] AND NOT (1 SHL Interrupts[ComPort]);

   End;



   Bufs[ComPort].Old_MCR:= Port[Bufs[ComPort].UART_MCR]; {Store MCR        }

   Port[Bufs[ComPort].UART_LCR]:= 3; {Parity to none and turn off the break}

   PORT[Bufs[ComPort].UART_IER]:= 1; {Enable data recieved interrupts      }



   InLine(Enable_Interrupts);



   Bufs[ComPort].Active:= True;      {Let COM_ISR know to check this port  }

End;



Procedure Com_Port.SetParam(Baud: Longint; WordSize: Byte;

                             Parity: Char; StopBits: Byte);



Const

      MaxBaud      = 115200;  {Maximum baud rate                            }



Var Divisor: Word;

    lcr    : Byte;



{Sets the baud rate, wordsize, parity, and stop bits used by the port.  The

 most common setting especially with high speed modems is 38400 baud, word

 size is 8 bits (one byte), 'n' or no parity, and one stop bit.  Compuserve

 uses 7 bits and even parity.}



Begin



   {This next section sets the baud rate based on the divisor of MAXBAUD}



   If Baud < 50 Then Baud:= 50;

   If Baud > MaxBaud Then Baud:= MaxBaud;

   Divisor:= MaxBaud Div Baud;

   InLine(Disable_Interrupts);

   Port [Bufs[CPort].uart_lcr ]:= Port[Bufs[Cport].uart_lcr] Or $80;

   Portw[Bufs[CPort].uart_Data]:= divisor;

   Port [Bufs[CPort].uart_lcr] := Port[Bufs[CPort].uart_lcr] And NOT $80;

   InLine(Enable_INterrupts);



   {This next section sets the parity}



   Case upcase(Parity) Of

      'n': lcr:= $00 or $03;

      'e': lcr:= $18 or $02;

      'o': lcr:= $08 Or $02;

      's': lcr:= $38 Or $02;

      'm': lcr:= $28 OR $02;

      Else

         Lcr:= $00 or $03;

   End;

   If StopBits = 2 Then lcr:= Lcr OR $04;



   InLine(Disable_Interrupts);

   Port[Bufs[CPort].Uart_lcr]:= Port[Bufs[CPort].uart_lcr] And $40 Or LCR;

   InLine(Enable_Interrupts);

End;



Procedure Com_Port.Done;



{Use this procedure when you are done with your program.  You >MUST< run

 this procedure for each COM variable you have initialized.  If you don't

 if any data comes in to the comm port dos will still try to go to the

 place where the com_isr >>>used<<< to be!  meaning your computer >could<

 crash.  since this is an oop approach exit-proc wasn't used since there

 could be any number of variables open and running.  Therefore it is YOUR

 responsibility to call this procedure for each COM_PORT object you have

 created}



Var InUse: Boolean;     {Scratch variable to test for shared interrupt}

    Ktr  : byte;        {Counter variable                             }



Begin

   {Check for shared interrupt usage}



   InUse:= False;

   For Ktr:= 1 to 4 Do

      If (Interrupts[Ktr] = Interrupts[CPort]) And Bufs[Ktr].Active Then

         InUse:= True;



   InLine(Disable_interrupts);



   {Restore the old Modem Control Register and disable incomming data

    interrupts}

   Port[Bufs[CPort].UART_MCR] := Bufs[CPort].Old_MCR;

   Port[Bufs[CPort].UART_IER] := 0;





   If Not InUse Then Begin

      {Remove the interrupt only if another object is not using it}

      Port[$21] := Port[$21] Or ($01 SHR Interrupts[CPort]);

      SetIntVec(8+Interrupts[CPort],Bufs[CPort].Org_Vector);

   End;



   InLine(Enable_Interrupts);



   CPort:= 0;                {Set CPort variable to 0     }



   {Release the buffer memory and set the active flag to false}

   Freemem(Bufs[CPort].R_Buffer,Bufs[CPort].R_Size);

   Freemem(Bufs[CPort].T_Buffer,Bufs[CPort].T_Size);

   Bufs[CPort].Active:= False;

End;



Function Com_Port.Read: Char;



{If a character is available on the buffer, this procedure will return the

 char.  If there is no character, char 0 (#0) will be returned.  If you

 expect #0 to be passed as part of the data call this routine only if

 function waiting returns true, or use readw - which will wait for

 a character from the com port if nothing is available}



Begin

   With Bufs[CPort] Do Begin

      If R_Head = R_Tail Then Begin  {Nothing in the buffer}

         Read:= #0;

      End Else Begin                 {Data waiting in the buffer}

         Read:= R_Buffer^[R_Head];          {Get the waiting character}

         Inc(R_Head);                       {Increment the queue pointer}

         If R_Head > R_Size Then R_Head:= 0; {Check for out of range}

      End;

   End;

End;



Function Com_Port.Readw: Char;



{Waits for a character from the comm port if none are available.  Passes

 back the first character it finds in the recieve buffer}



 Begin

    With Bufs[CPort] Do Begin

       While R_Head = R_Tail Do;

       Readw:= R_Buffer^[R_Head];

       Inc(R_Head);

       If R_Head > R_Size THen R_Head:= 0;

    End;

 End;



Function Com_Port.Waiting: Boolean;



{This function returns TRUE if there is data waiting in the recieve buffer}



Begin

   Waiting:= (Bufs[CPort].R_Head <> Bufs[CPort].R_Tail);

End;



Procedure Com_Port.Enable;



{This should be the third command you run after .INIT and .SETPARAM.  Enable

 and Disable are provided for multi-port use.  If you are using com1 and

 com2 together you can leave both ports enabled at the same time, likewise

 with com3 and com4.  However ports that share interrupts (Com1 & Com3)

 (com2 & com4) can not both be enabled at the same time.  This is the reason

 I wrote this unit because the most popular pascal com libraries

 (particularly the async libraries by rising sun which are otherwise

 extrodinarilly compitent packages) could not handle shared interrupts.

 This unit can, but it cheats by allowing you to "suspend" one of the ports

 on the shared interrupt.  While a port is suspended (disabled) you can not

 send or recieve data from that port.  Other considerations - a mouse on

 com1 will not work with this package when you use com3, likewise a mouse

 on com2 will not work when you run com4 this is because this package

 installs it's own interrupts - overwriting the mouse ports (although the

 mouse will begin working again once you call .done}



begin

   inline(disable_interrupts);

   port[bufs[cport].uart_mcr]:= 11;

   inline(enable_interrupts);

end;



procedure com_port.disable;



{call this procedure only if you are about to enable another port which

 uses the same interrupt, see com_port.enable for more information}



begin

   inline(disable_interrupts);

   port[bufs[cport].uart_mcr]:= 3;

   inline(enable_interrupts);

end;



procedure com_port.write(c: char);



{this procedure places a character on the transmit buffer}



begin

   with bufs[cport] do begin

      t_buffer^[t_tail]:= c;

      inc(t_tail);

      if t_tail > t_size then t_tail:=0;

      if (t_tail = t_head) then begin

         inc(t_head); {overflow}

         if t_head > t_size then t_head:=0;

      end;

      inline(disable_interrupts);

      {tell the modem to alert us when it is ok to send data}

      port[uart_ier]:= port[uart_ier] or 2;

      inline(enable_interrupts);

   end;

end;



procedure com_port.writes(s: string);



{passes a string to the transmit buffer}



var ktr: byte;



begin

   for ktr:= 1 to length(s) do

      write(s[ktr]);

end;



procedure com_port.break;



{this procedure sends a >break< signal down the line.  it is usefull

 primarilly for mainframe and unix based systems, dos based machines

 do not look for or respond to the break signal.}



var

   org_data: byte;



begin

   inline(disable_interrupts);

   with bufs[cport] do begin

      org_data:= port[uart_lcr];   {save the contents of the lcr            }

      port[uart_lcr]:= 255;        {load up the line control register       }

      delay(3);                    {crt unit - delay 3 thousands of a second}

      port[uart_lcr]:= org_data;   {restore the line control register       }

   end;

   inline(enable_interrupts);

end;



function com_port.online: boolean;



{this function returns true if the modem status register indicates a data

 carrier detect signal.  note that some modems always return true even when

 not connected, an at command is needed to force dcd to return the true state

 of the modem.  also note that some direct serial connections (i.e. no modem

 but hardwired to another machine, may not return the correct dcd stat or

 may be false even when connected - this is particularly true of three wire

 direct serial connections (pins 2, 3, and 7 wired all others unwired)}



begin

    online := (port[bufs[cport].uart_msr] and $80) > 0;

end;



procedure com_port.hangup;



{this procedure disconects the modem by lowering the dtr signal.  note that

 some modems may not be affected by this procedure based on their at

 configurations.  direct serial lines are not usually affected by this

 signal.  your best bet is to issue this command then send '+++' to the modem

 and wait five seconds and then issue 'ATH<return>'}



var org_mcr: byte;  {scratch var to store the original mcr stuff}



begin

  with bufs[cport] do begin

     org_mcr := port[uart_mcr];

     port[uart_mcr]:= org_mcr or $fe;  {lower the dtr signal  }

     delay(100);                       {delay 100 ms          }

     port[uart_mcr]:= org_mcr;         {restore the dtr signal}

  end;

end;



var ktr: byte;



begin



   {initialize the 4 comm buffers to an inactive state, this code is run

    the moment you start the program, automatically}



   for ktr:= 1 to 4 do begin

      bufs[ktr].active  := false;

      bufs[ktr].r_buffer:= nil;

      bufs[ktr].r_head  := 0;

      bufs[ktr].r_tail  := 0;

      bufs[ktr].r_size  := 0;

      bufs[ktr].t_buffer:= nil;

      bufs[ktr].t_head  := 0;

      bufs[ktr].t_tail  := 0;

      bufs[ktr].t_size  := 0;

   end;

end.



{this area is ignored by turbo pascal}



sample program:



program sample;



uses async,crt;



var

   c: char;

   com1: com_port;



begin

   com1.init(1,10000,10000);            {set up the buffers & interrupt    }

   com1.setparam(38400,8,'N',1);        {set up the baud,ws,parity,&stopbts}

   com1.enable;                         {enable the com port               }

   c:= ' ';                             {initialize the scratch variable   }

   writeln('Press ESC to exit dumb terminal');

   loop

      if keypressed then begin

        c:= readkey;

        if c <> #27 then com1.write(c);

     end;

     if com1.waiting then write(com1.read);

  until c = #27;

  com1.done;

end.

}



{an explination of the uart data areas   the port address is offset by the

 numbers.  so if you're on com1 and com1 as at 3f8, the uart_IER address is

 at port address 3f8+1, uart_iir is 3f8+2, etc}



      uart_data    = 0;       {Uart Data Offset                             }

                                 {Transmit/Recieve Data area                }

      uart_ier     = 1;       {Uart Interrupt Enable Register               }

                                 {Bits 7-4 always 0                         }

                                 {Bit    3  1 = enable change in modem stat }

                                 {Bit    2  1 = enable line-status interrupt}

                                 {Bit    1  1 = enable transmit reg empty   }

                                 {Bit    0  1 = data available interrupt    }

      uart_iir     = 2;       {Uart Interrupt Identification Register       }

                                 {Bits 7-3 always 0                         }

                                 {Bits 2-1 01 = transmit - register empty   }

                                 {         10 = data available              }

                                 {         11 = line status                 }

                                 {Bit    0  1 = No Interrupt pending        }

                                 {          0 = Interrupt Pending           }

      uart_lcr     = 3;       {Uart Line Control Register}

                                 {Bit    7  0 = Normal, 1=Address Baud rate }

                                 {Bit    6  0 = break disabled, 1 enabled   }

                                 {Bit    5  0 = Don't force parity          }

                                 {          1 = if bit 4-3 = 01 parity = 1  }

                                 {              if bit 4-3 = 11 parity = 0  }

                                 {              if bit   3 = 0 no parity    }

                                 {bit    4  0 = odd parity,1=even parity    }

                                 {bit    3  0 = no parity, 1=parity         }

                                 {bit    2  0 = 1 stop bit                  }

                                 {          1 = 1.5 stop bits if 5bits/char }

                                 {              or 2 stop bits if 6-8 bits  }

                                 {bits 1-0 00 = 5 bits/character            }

                                 {         01 = 6 bits/character            }

                                 {         10 = 7 bits/character            }

                                 {         11 = 8 bits/character            }

      uart_mcr     = 4;       {uart modem control register                  }

                                 {bits 7-5 always 0                         }

                                 {bit    4  0=normal,1=loop back test       }

                                 {bit    3  1=interrupts to system bus      }

                                 {bit    2  user designated output          }

                                 {bit    1  1=active rts                    }

                                 {bit    0  1=active dtr                    }

      uart_lsr     = 5;       {uart line status register                    }

                                 {bit    7  always 0                        }

                                 {bit    6  1=transmit shift reg is empty   }

                                 {bit    5  1=transmit hold reg is empty    }

                                 {bit    4  1=break recieved                }

                                 {bit    3  1=framing error recieved        }

                                 {bit    2  1=parity error recieved         }

                                 {bit    1  1=overrun error recieved        }

                                 {bit    0  1=data received                 }

      uart_msr     = 6;       {uart modem status register                   }

                                 {bit    7  1=recieve line signal detect    }

                                 {bit    6  1=ring indicator                }

                                 {bit    5  1=data signal ready             }

                                 {bit    4  1=clear to send                 }

                                 {bit    3  1=recieve line signal change    }

                                 {bit    2  1=ring indicator has changed    }

                                 {bit    1  1=dsr has changed state         }

                                 {bit    0  1=cts has changed state         }

